import useChannelsTabWazzupExtensionStore from './../../store/useChannelsTabWazzupExtensionStore.js';

export const CurrentSelectedTable = {
  name: 'current-selected-table',
  props: {
    selected: {
      type: String,
      default: ""
    },
    channelId: {
      type: String,
      default: ""
    },
  },

  setup(props) {
    const channelStore = useChannelsTabWazzupExtensionStore();
    const metaTables = Vue.inject('metaTables');
    const metaFields = Vue.inject('metaFields');
    const metaFilters = Vue.inject('metaFilters');

    const tableId = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0];
      return currentTable?.table_id || 0;
    });

    const fullName = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0];
      return currentTable?.client_name_field_id || 0;
    });

    const responsibleManager = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0];
      return currentTable?.responsible_manager_field_id || 0;
    });

    const phoneFieldId = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0];
      return currentTable?.phone_field_id || 0;
    });

    const usernameFieldId = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0];
      return currentTable?.username_field_id || 0;
    });

    const priority = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0] || 'low';
      return currentTable?.priority || 'low';
    });

    const filter = Vue.computed(() => {
      let temp = channelStore.channels[props.channelId]['additional_option'];
      let currentTable = temp.filter(i => i.id == props.selected)[0];
      return currentTable?.filter || 0;
    });

    Vue.watch(tableId, _ => setTimeout(_ => $("select.chosen_add").trigger("chosen:updated"), 0));

    // Инициализация chosen с функцией при change
    function initChosen() {
      $('.chosen_add').chosen().change(function(event) {
        let selectAttr = event.target.getAttribute('data-feature');
        let selectValue = event.target.value;
        setValue(selectAttr, selectValue);
      });
    };

    function setValue(key, value) {
        channelStore.channels[props.channelId]['additional_option'].forEach(i => {
          if (i.id == props.selected) {
            i[key] = value;
          }
        });
      }

    Vue.onMounted(() => {
      checkHelpButtonsOnCurrentPage();
      bind_help_bt('wazzup');
      initChosen();
    });

    return {
      channelStore,
      metaTables, metaFields, metaFilters,
      tableId, usernameFieldId, phoneFieldId, priority, filter, fullName, responsibleManager,

      setValue,

      getFilters() {
        let t_id = Number(tableId.value);
        if (t_id <= 0) {
          return {};
        }

        let arr = [];
        Object.keys(metaFilters[t_id]).forEach(i => {
          arr.push({id:metaFilters[t_id][i].id, name:metaFilters[t_id][i].name});
        });
        return arr;
      },
    }
  },

  template: /*html*/`
    <div class="current-selected-table">
      <div class="current-selected-table__name">
        <span>{{ $t('Table') }}</span>
        <span h_id="wazzup_channel_table" class="help_bt"></span>
        <span>:</span>
      </div>
      <div class="current-selected-table__select">
        <select
          class="form-control chosen_add"
          v-model="tableId"
          data-feature="table_id"
        >
          <option value="0"></option>
          <template v-for="table in metaTables">
            <option :value="table.id">{{ table.name }}</option>
          </template>
        </select>
      </div>

      <div class="current-selected-table__name">
        <span>{{ $t('priority') }}</span>
        <span h_id="wazzup_channel_priopity" class="help_bt"></span>
        <span>:</span>
      </div>
      <div class="current-selected-table__select">
        <select
          class="form-control"
          v-model="priority"
          @change="setValue('priority', $event.target.value)"
        >
          <template v-for="pr in ['low','medium','high']">
            <option :value="pr">{{ $t(pr) }}</option>
          </template>
        </select>
      </div>

      <div class="current-selected-table__name">
        <span>{{ $t('client_name_field_label') }}</span>
        <span h_id="wazzup_channel_client_name" class="help_bt"></span>
        <span>:</span>
      </div>
      <div class="current-selected-table__select">
        <select
          class="form-control chosen_add"
          v-model="fullName"
          data-feature="client_name_field_id"
        >
          <option value="0"></option>
          <template v-for="field in metaFields[tableId] || []">
            <option
              v-if="field.type == 3"
              :value="field.id"
            >{{ field.name }}</option>
          </template>
        </select>
      </div>

      <div class="current-selected-table__name">
        <span>{{ $t('responsible_manager_field_label') }}</span>
        <span h_id="wazzup_channel_responsible_manager" class="help_bt"></span>
        <span>:</span>
      </div>
      <div class="current-selected-table__select">
        <select
          class="form-control chosen_add"
          v-model="responsibleManager"
          data-feature="responsible_manager_field_id"
        >
          <option value="0"></option>
          <template v-for="field in metaFields[tableId] || []">
            <option
              v-if="field.type == 7 || field.type == 11"
              :value="field.id"
            >{{ field.name }}</option>
          </template>
        </select>
      </div>

      <div class="current-selected-table__name">
        <span>{{
          channelStore.channels[this.channelId]?.transport == 'instagram'
          ? $t('nickname_field')
          : $t('phone_field')
        }}</span>
        <span h_id="wazzup_channel_field" class="help_bt"></span>
        <span>:</span>
      </div>
      <div class="current-selected-table__select">
        <select
          class="form-control chosen_add"
          v-model="phoneFieldId"
          data-feature="phone_field_id"
        >
          <option value="0"></option>
          <template v-for="field in metaFields[tableId] || []">
            <option :value="field.id">{{ field.name }}</option>
          </template>
        </select>
      </div>

      <template v-if="channelStore.channels[this.channelId]?.transport === 'tgapi'">
        <div class="current-selected-table__name">
          <span>{{ $t('nickname_field') }}</span>
          <span h_id="wazzup_channel_field_username" class="help_bt"></span>
          <span>:</span>
        </div>
        <div class="current-selected-table__select">
          <select
            class="form-control chosen_add"
            v-model="usernameFieldId"
            data-feature="username_field_id"
          >
            <option value="0"></option>
            <template v-for="field in metaFields[tableId] || []">
              <option :value="field.id">{{ field.name }}</option>
            </template>
          </select>
        </div>
      </template>

      <div class="current-selected-table__name">
        <span>{{ $t('filter') }}</span>
        <span h_id="wazzup_channel_filter" class="help_bt"></span>
        <span>:</span>
      </div>
      <div class="current-selected-table__select">
        <select
          class="form-control chosen_add"
          v-model="filter"
          data-feature="filter"
        >
          <option value="0"></option>
          <template v-for="filter in getFilters()">
            <option :value="filter.id">{{ filter.name }}</option>
          </template>
        </select>
      </div>
    </div>
  `
}
