import useApiTabWazzupExtensionStore from './useApiTabWazzupExtensionStore.js';

/**
 * Получить название канала по умолчанию.
 * @param {string} name
 * @returns {string|null}
 */
function getDefaultName(name) {
  switch (name) {
    case 'whatsapp':
      return 'Whatsapp';
    case 'instagram':
      return 'Instagram';
    case 'tgapi':
      return 'Telegram';
    default:
      return null;
  }
}

export default Pinia.defineStore('useChannelsTabWazzupExtensionStore', {
  state: () => ({
    channels: {}
  }),

  actions: {
    getChannels() {
      const apiStore = useApiTabWazzupExtensionStore();

      fetch("https://api.wazzup24.com/v3/channels", {
        method: 'GET',
        redirect: 'follow',
        headers: {
          "Authorization": `Bearer ${apiStore.api_key}`,
          "Content-Type": "application/json"
        },
      })
      .then(response => response.json())
      .then(result => {
        if (result instanceof Array) {
          result.forEach(item => {
            if (
              (
                item.transport == "whatsapp" ||
                item.transport == "instagram" ||
                item.transport == "tgapi"
              ) &&
              this.channels[item.channelId] == undefined
            ) {
              let chan = {};
              chan[item.channelId] = {
                "channel_id": item.channelId,
                "plain_id": item.plainId,
                "transport": item.transport,
                "channel_name": getDefaultName(item.transport),
                "additional_option": []
              };

              this.channels = { ...this.channels, ...chan };
            }
          });
        }
      });
    },

    deleteChannel(id) {
      this.channels[id]['deleted'] = true;
    },
  }
});
