import { CaptionComponent } from "./CaptionComponent.js";
import { SearchComponent } from "./SearchComponent.js";
import { TableComponent } from "./TableComponent.js";


const { inject, ref, onMounted } = Vue;

export const App = {
  components: {
    'caption-component': CaptionComponent,
    'search-component':  SearchComponent,
    'table-component':   TableComponent,
  },

  setup() {
    const block_id = inject('block_id');
    const block_name = inject('block_name');
    const report_id  = inject('report_id');

    const preloader = ref(false);
    const search_value = ref('');
    const data = ref([]);
    const last_lines = ref([]);

    async function search(value, need_last_lines) {
      if (value == '') {
        search_value.value = value;
        data.value = [];
        return;
      }

      if (!need_last_lines) {
        preloader.value = true;
        search_value.value = value;
      }

      const form_data = new FormData();
      form_data.append('csrf',  csrf);
      form_data.append('value', value);
      form_data.append('get_json_data', block_id);
      if (need_last_lines) {
        form_data.append('last_lines', 'last_lines');
      }

      let url = '';
      if (report_id === null) { // отчет по внешней ссылке
        const params_from_url = new URLSearchParams(window.location.search);
        url = 'report_external.php?hash=' + params_from_url.get('hash');
      } else {
        url = `report.php?id=${report_id}`;
      }

      let response = await fetch(url, {
        method: 'POST',
        mode: 'cors',
        credentials: 'include',
        body: form_data
      });

      let result = await response.json();
      // console.log(result[0]);

      if (!need_last_lines) {
        last_lines.value = [];
        data.value = result[0] || [];

        if (typeof window['search_block_values'][block_name] === 'undefined') {
          window['search_block_values'][block_name] = '';
        }

        window['search_block_values'][block_name] = value;

        setTimeout(_ => preloader.value = false, 200);
      } else {
        last_lines.value = result[0] || [];
        setTimeout(_ => preloader.value = false, 200);
      }
    }

    onMounted(_ => search('last_lines', true));

    return {
      preloader,
      search_value,
      data,
      last_lines,
      getTranslate: (key) => lang[key] || key,
      search
    }
  },

  template: /*html*/`
    <caption-component />
    <search-component @handle-change='search'></search-component>

    <div
      v-if='preloader'
      style="text-align: center; margin-top: 20px;"
    >{{ getTranslate('Sync_pas_m') }}...</div>
    <template v-else-if='last_lines.length > 0'>
      <div class="repconview__table_table">
        <div
          class="repconview__table_inner"
          style="overflow: auto hidden;"
        >
          <table-component :rows='last_lines'></table-component>

          <div
            class="repconview__table-legend"
            style="overflow: hidden;"
          >
            {{ getTranslate('Total_lines') + ' ' + last_lines.length }}
          </div>
        </div>
      </div>
    </template>
    <template v-else>
      <div class="repconview__table_table">
        <div
          class="repconview__table_inner"
          style="overflow: auto hidden;"
        >
          <table-component :rows='data'></table-component>

          <div
            v-if='search_value != ""'
            class="repconview__table-legend"
            style="overflow: hidden;"
          >
            {{ getTranslate('Lines_found') + ' ' + data.length }}
          </div>
        </div>
      </div>
    </template>
  `
}
