const { inject, ref, onMounted } = Vue;


const FIELD_NUMBER = 1; // поле типа "число"
const FIELD_DATE   = 2; // поле типа "дата"
const FIELD_TEXT   = 3; // поле типа "текст"
const FIELD_LINK   = 5; // поле типа "связь"
const SYSTEM_FIELD_DATE = 12; // системное поле "Время добавления" field_type_value


// Компонент представления ячеек
export const CellViewComponent = {
  props: ['column', 'row'],

  setup(props, {emit}) {
    const table_id  = inject('table_id');
    const use_links = inject('use_links');
    const cut_to    = inject('cut_to');

    const field_type = Number(props.column['field_type'] || 0);
    const field_type_value = props.column['field_type_value'] || 0;
    const value = props.row[props.column.column_name] || '';

    const text = ref('');

    onMounted(_ => text.value = buildText());

    // Сформировать текст ссылки
    function buildText() {
      let result = '';

      switch (field_type) {
        case FIELD_NUMBER:
          return (Number(value)).toLocaleString('ru-RU', {
            style: 'currency',
            currency: 'RUB',
          }).slice(0, -2);

        case FIELD_DATE:
        case SYSTEM_FIELD_DATE:
          return value == '0000-00-00 00:00:00' ? '' : (
            field_type_value == '0'
              ? moment(value).format('DD.MM.Y')       // без времени
              : moment(value).format('DD.MM.Y hh:mm') // с временем
          );

        case FIELD_TEXT:
          result = value; break;

        default:
          return value;
      }

      if (cut_to > 0) {
        result = result.slice(0, cut_to);
        result = result.length > 0 ? `${result}...` : '';
      }
      return result;
    }

    // Сформировать класс тэга а
    function buildClassName() {
      if (use_links == 'implicit') { // Показывать скрытые ссылки
        return 'repconview__implicit_link';
      } else if (use_links == 'explicit') { // Показывать явные ссылки
        return 'repconview__explicit_link';
      } else {
        return '';
      }
    }

    // Сформировать стили тэга а
    function buildStyle() {
      if (use_links == 'implicit') { // Показывать скрытые ссылки
        return 'text-decoration: none;';
      } else if (use_links == 'explicit') { // Показывать явные ссылки
        return 'color:rgb(0,0,0);';
      } else {
        return '';
      }
    }

    // Сформировать url тэга а
    function buildHref() {
      let linked_table_id = props.column['field_type_value'].split('|', 1) || 0;
      const href_linkid = props.row[props.column['column_name'] + '_href_linkid'] || 0;
      const href_rowid = props.row[props.column['column_name'] + '_href_rowid'] || 0;

      switch (field_type) {
        case FIELD_LINK:
          return 'view_line2.php?table=' + linked_table_id + '&line=' + href_linkid;

        default:
          return 'view_line2.php?table=' + table_id + '&line=' + href_rowid;
      }
    }

    return {
      use_links,
      buildClassName,
      buildStyle,
      buildHref,

      text
    }
  },

  template: /*html*/`
    <span v-if='use_links == "none"'>{{ text }}</span>
    <a
      v-else
      :class="buildClassName()"
      :style="buildStyle()"
      :href="buildHref()"
      target="__blank"
    >{{ text }}</a>
  `
}
