const { inject, ref, onMounted } = Vue;

export const SearchComponent = {
  emits: ['handleChange'],

  setup(props, {emit}) {
    const block_name = inject('block_name');

    const def_val = typeof window['search_block_values'] === 'undefined' ? '' : (
      typeof window['search_block_values'][block_name] === 'undefined'
        ? '' : window['search_block_values'][block_name]
    );

    const searchString = ref(def_val);
    onMounted(() => {
      if (searchString.value != '') {
        emit("handleChange", searchString.value);
      }
    });

    return {
      /** @param {string} Поисковая строка */
      searchString,
      getTranslate: () => lang['Search'] || 'Search',

      // https://lodash.com/docs/4.17.15#debounce
      updateValue: _.debounce(value => {
        emit("handleChange", value);
        setTimeout(() => $('.repconview__table_table').fixedTable(), 500);
      }, 500),
    }
  },

  template: /*html*/`
    <div style="text-align: right; display: flex; align-items: center;">
      <input
        v-model='searchString'
        @input='updateValue(searchString)'
        @keyup.enter='updateValue(searchString)'
        class='form-control'
        type='search'
        :placeholder='getTranslate()'
        style='margin: 3px;'
      />

      <button
        type="button"
        style="margin-left: 5px; filter: var(--svg-filter-gray-on-white);"
        class="header__user-item--search"
        :title="getTranslate()"
        @click='updateValue(searchString)'
      ></button>
    </div>
  `
}
